@@ -1,11 +1,11 @@
 package Rope;
 
 use 5.006; use strict; use warnings;
-our $VERSION = '0.12';
-
+our $VERSION = '0.13';
 use Rope::Object;
 my (%META, %PRO);
 our @ISA;
+
 BEGIN {
 	%PRO = (
 		keyword => sub {
@@ -14,12 +14,19 @@ BEGIN {
 			*{"${caller}::${method}"} = $cb;
 		},
 		scope => sub {
-			my ($self, %props) = @_;
+			my ($caller, $self, %props) = @_;
 			for (keys %{$props{properties}}) {
 				$props{properties}->{$_} = {%{$props{properties}{$_}}};
 				if ($props{properties}{$_}{value} && ref $props{properties}{$_}{value} eq 'CODE') {
 					my $cb = $props{properties}{$_}{value};
-					$props{properties}{$_}{value} = sub { $cb->($self, @_) };
+					$props{properties}{$_}{value} = sub { $cb->($META{initialised}{${$self}->{identifier}}, @_) };
+				}
+				my $trigger = $props{properties}{$_}{trigger};
+				if (defined $trigger) {
+					$props{properties}{$_}{trigger} = sub { 
+						$trigger->($META{initialised}{${$self}->{identifier}}, @_) 
+
+					};
 				}
 			}
 			return \%props;
@@ -35,12 +42,13 @@ BEGIN {
 		set_prop => sub {
 			my ($caller, $prop, %options) = @_;
 			if ($META{$caller}{properties}{$prop}) {
+				defined $options{$_} && do { $META{$caller}{properties}{$prop}{$_} = $options{$_} } for qw/builder trigger/;
 				if ($META{$caller}{properties}{$prop}{writeable}) {
-					$META{$caller}{properties}{$prop}{value} = $options{value};
+					$META{$caller}{properties}{$prop}{value} = $options{value} if exists $options{value};
 					$META{$caller}{properties}{$prop}{class} = $caller;
 				} elsif ($META{$caller}{properties}{$prop}{configurable}) {
 					if ((ref($META{$caller}{properties}{$prop}{value}) || "") eq (ref($options{value}) || "")) {
-						$META{$caller}{properties}{$prop}{value} = $options{value};
+						$META{$caller}{properties}{$prop}{value} = $options{value} if exists $options{value};
 						$META{$caller}{properties}{$prop}{class} = $caller;
 					} else {
 						die "Cannot inherit $META{$caller}{properties}{$prop}{class} and change property $prop type";
@@ -87,7 +95,7 @@ BEGIN {
 					my ($prop, $options) = (shift @properties, shift @properties);
 
 					my $ref = ref $options;					
-					if (!$ref || $ref ne 'HASH' || ! grep { defined $options->{$_} } qw/initable writeable builder enumerable configurable value/) {
+					if (!$ref || $ref ne 'HASH' || ! grep { defined $options->{$_} } qw/initable writeable builder enumerable configurable trigger value/) {
 						$options = {
 							initable => 1,
 							enumerable => 1,
@@ -225,12 +233,24 @@ BEGIN {
 				}
 			};
 		},
+		destroy => sub {
+			my ($caller) = shift;
+			return sub {
+				my ($self, $locked) = @_;
+				if (ref $self) {
+					delete $META{initialised}{${$self}->{identifier}};
+					$self->DESTROY;
+					return;
+				}
+			};
+		},
 		new => sub {
 			my ($caller) = shift;
 			return sub {
 				my ($class, %params) = @_;
 				my $self = \{
 					prototype => {},
+					identifier => $META{initialised}{identifier}++
 				};
 				$self = bless $self, $caller;
 				my $build = $PRO{clone}($META{$caller});
@@ -252,14 +272,16 @@ BEGIN {
 						};
 					}
 				}
+
 				for ( sort { $build->{properties}->{$a}->{index} <=> $build->{properties}->{$b}->{index} } keys %{ $build->{properties} } ) {
 					if ( !defined $build->{properties}->{$_}->{value} && defined $build->{properties}->{$_}->{builder}) {
 						my $builder = $build->{properties}->{$_}->{builder};
 						$build->{properties}->{$_}->{value} = ref $builder ? $builder->($build) : $caller->$builder($build);
 					}
 				}
-				tie %{${$self}->{prototype}}, 'Rope::Object', $PRO{scope}($self, %{$build});
-
+				tie %{${$self}->{prototype}}, 'Rope::Object', $PRO{scope}($caller, $self, %{$build});
+				$META{initialised}->{${$self}->{identifier}} = $self;
+				$self->{ROPE_init}->();
 				return $self;
 			};
 		}
@@ -288,7 +310,7 @@ sub import {
 	$PRO{keyword}($caller, $_, $PRO{$_}($caller))
 		for $options->{import} 
 			? @{$options->{import}} 
-			: qw/function property properties prototyped extends with requires locked new/;
+			: qw/function property properties prototyped extends with requires locked destroy new/;
 }
 
 sub new {
@@ -334,7 +356,7 @@ Rope - Tied objects
 
 =head1 VERSION
 
-Version 0.12
+Version 0.13
 
 =cut
 
@@ -345,6 +367,7 @@ Version 0.12
 	use Rope;
 
 	prototyped (
+		bend_update_count => 0,
 		loops => 1,
 		hitches => 10,
 		...
@@ -358,7 +381,12 @@ Version 0.12
 			initable => 1,
 			configurable => 1,
 			enumerable => 1,
-			required => 1
+			required => 1,
+			trigger => sub {
+				my ($self, $value) = @_;
+				$self->{bend_update_count}++;
+				return $value;
+			}
 		},
 		...
 	);
@@ -417,6 +445,12 @@ The type property/key expects a code ref to be passed, all values that are then
 
 The buidler property/key expects either a code ref or a scalar that represents the name of a sub routine in your class (currently not functions/properties but may extend in the future). It expects the value for that property to be returned from either the code ref or sub routine. Within a builder you can also add new properties to your object by extending the passed defintion, when extending this way I would suggest using ++$_[0]->{keys} to set the index so that sorting is persistent further down the line.
 
+=head2 trigger
+
+The trigger property/key expects a code ref that expects the value for that property to be returned so it can also be used for coecion on setting of a property. Two values are passed into a trigger the first being $self which allows you to access and manipulate the existing objects other properties, the second is the value that is being set for the property being called.
+
+A trigger is also called on DELETE, in this case you will get three params, the second being the text 'DELETE', the third being the last set value.
+
 =head2 index
 
 The index property/key expects an integer, if you do not set then this integer it's automatically generated and associated to the property. You will only want to set this is you always want to have a property last when itterating
@@ -471,7 +505,7 @@ Extends the current object definition with a new property that acts as a functio
 		...
 	};
 
-NOTE: normal sub tradditional routines work and should be inherited also.	
+NOTE: traditional sub routines work and should be inherited also.	
 
 	sub three {
 		my ($self, $param) = @_;
@@ -526,7 +560,7 @@ The locked keyword can be used to lock your Object so that it can no longer be e
 
 	1;
 
-Once you have an object initialised you can toggle whether is locked by calling 'locked' with either a true or false value.
+Once you have an object initialised you can toggle whether it is locked by calling 'locked' with either a true or false value.
 
 	$obj->locked(1);
 
@@ -565,6 +599,12 @@ Along with class definitions you can also generate object using Rope itself, the
 	$knot->{loops};
 	$with->loops;
 
+=head2 destroy
+
+For objects that are running in a long process, like under a web server, you will want to explicitly call destroy on your objects as to achieve the correcting scoping a reference has to be held in memory. If running under a script the problem will not exists as destruction happens when that script ends.
+
+	$knot->destroy();
+
 =head1 OBJECT CLASS DEFINITION
 
 	package Builder;
