@@ -6,13 +6,21 @@ use warnings;
 sub TIEHASH {
         my ($class, $obj) = @_;
         my $self = bless $obj || {}, $class;
+	$self->{properties}->{ROPE_init} = {
+		value => sub { $self->init }
+	};
+	return $self;
+}
+
+sub init {
+	my ($self) = @_;
 	$self->set_value(
 		$_,
 		$self->{properties}->{$_}->{value},
 		$self->{properties}->{$_}
 	) for keys %{$self->{properties}};
 	$self->compile();
-	return $self;
+	delete $self->{properties}->{ROPE_init};
 }
 
 sub compile {
@@ -34,6 +42,9 @@ sub compile {
 sub set_value {
 	my ($self, $key, $value, $spec) = @_;
 	if (defined $value) {
+		if ($spec->{trigger}) {
+			$value = $spec->{trigger}->($value);
+		}
 		if ($spec->{type}) {
 			$value = eval {
 				 $spec->{type}->($value);
@@ -111,7 +122,10 @@ sub EXISTS {
  
 sub DELETE { 
         my $k = $_[0]->{properties}->{$_[1]};
-        my $del = !$_[0]->{locked} && $k->{writeable} ? delete $_[0]->{properties}->{$_[1]} : undef;
+	if ($k->{trigger}) {
+		$k->{trigger}->('DELETE', $k->{value});
+	}
+	my $del = !$_[0]->{locked} && $k->{writeable} ? delete $_[0]->{properties}->{$_[1]} : undef;
 	$_[0]->compile() if $del;
 	return $del;
 }
@@ -125,6 +139,8 @@ sub SCALAR {
         scalar keys %{$_[0]->{properties}}
 }
 
+sub DESTROY { }
+
 1;
 
 __END__ 
